@@ -1,11 +1,11 @@
-;;; editorconfig.el --- EditorConfig Emacs Plugin
+;;; editorconfig.el --- EditorConfig Emacs Plugin  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2011-2017 EditorConfig Team
+;; Copyright (C) 2011-2019 EditorConfig Team
 
 ;; Author: EditorConfig Team <editorconfig@googlegroups.com>
-;; Version: 0.7.14
+;; Version: 0.8.1
 ;; URL: https://github.com/editorconfig/editorconfig-emacs#readme
-;; Package-Requires: ((cl-lib "0.5"))
+;; Package-Requires: ((cl-lib "0.5") (emacs "24"))
 
 ;; See
 ;; https://github.com/editorconfig/editorconfig-emacs/graphs/contributors
@@ -61,9 +61,9 @@ coding styles between different editors and IDEs."
 
 (defcustom editorconfig-exec-path
   "editorconfig"
-  "EditorConfig executable name.
+  "Path to EditorConfig executable.
 
-This executable is invoked by `editorconfig-call-editorconfig-exec'."
+Used by `editorconfig-call-editorconfig-exec'."
   :type 'string
   :group 'editorconfig)
 (define-obsolete-variable-alias
@@ -77,8 +77,27 @@ This executable is invoked by `editorconfig-call-editorconfig-exec'."
 
 This function will be called with no argument and should return a
 hash object containing properties, or nil if any core program is
-not available.  The hash object should have symbols of property
-names as keys and strings of property values as values."
+not available.  Keys of this hash should be symbols of properties, and values
+should be strings of their values.
+
+
+For example, if you always want to use built-in core library instead
+of any EditorConfig executable to get properties, add following to
+your init.el:
+
+  (set-variable 'editorconfig-get-properties-function
+                #'editorconfig-core-get-properties-hash)
+
+Possible known values are:
+
+* `editorconfig-get-properties' (default)
+  * Use `editorconfig-get-properties-from-exec' when
+    `editorconfig-exec-path' executable executable is found, otherwise
+    use `editorconfig-core-get-properties-hash'
+* `editorconfig-get-properties-from-exec'
+  * Get properties by executing EditorConfig executable
+* `editorconfig-core-get-properties-hash'
+  * Always use built-in Emacs-Lisp implementation to get properties"
   :type 'function
   :group 'editorconfig)
 (define-obsolete-variable-alias
@@ -87,8 +106,7 @@ names as keys and strings of property values as values."
   "0.5")
 
 (defcustom editorconfig-mode-lighter " EditorConfig"
-  "Lighter displayed in mode line
-when `editorconfig-mode' is enabled."
+  "`editorconfig-mode' lighter string."
   :type 'string
   :group 'editorconfig)
 
@@ -139,9 +157,10 @@ overwrite \"indent_style\" property when current `major-mode' is a
   :group 'editorconfig)
 
 (defcustom editorconfig-indentation-alist
-  ;; For contributors: Sort modes in alphabetical order, please :)
+  ;; For contributors: Sort modes in alphabetical order
   '((apache-mode apache-indent-level)
     (awk-mode c-basic-offset)
+    (bpftrace-mode c-basic-offset)
     (c++-mode c-basic-offset)
     (c-mode c-basic-offset)
     (cmake-mode cmake-tab-width)
@@ -151,8 +170,16 @@ overwrite \"indent_style\" property when current `major-mode' is a
     (csharp-mode c-basic-offset)
     (css-mode css-indent-offset)
     (emacs-lisp-mode lisp-indent-offset)
+    (enh-ruby-mode enh-ruby-indent-level)
     (erlang-mode erlang-indent-level)
     (ess-mode ess-indent-offset)
+    (f90-mode f90-associate-indent
+              f90-continuation-indent
+              f90-critical-indent
+              f90-do-indent
+              f90-if-indent
+              f90-program-indent
+              f90-type-indent)
     (feature-mode feature-indent-offset
                   feature-indent-level)
     (fsharp-mode fsharp-continuation-offset
@@ -167,6 +194,7 @@ overwrite \"indent_style\" property when current `major-mode' is a
                   haskell-indentation-where-post-offset
                   haskell-indentation-where-pre-offset
                   shm-indent-spaces)
+    (haxor-mode haxor-tab-width)
     (idl-mode c-basic-offset)
     (jade-mode jade-tab-width)
     (java-mode c-basic-offset)
@@ -177,12 +205,15 @@ overwrite \"indent_style\" property when current `major-mode' is a
     (js3-mode js3-indent-level)
     (json-mode js-indent-level)
     (julia-mode julia-indent-offset)
-    (latex-mode . editorconfig-set-indentation/latex-mode)
+    (kotlin-mode kotlin-tab-width)
+    (latex-mode . editorconfig-set-indentation-latex-mode)
     (lisp-mode lisp-indent-offset)
     (livescript-mode livescript-tab-width)
     (lua-mode lua-indent-level)
     (matlab-mode matlab-indent-level)
+    (mips-mode mips-tab-width)
     (mustache-mode mustache-basic-offset)
+    (nasm-mode nasm-basic-offset)
     (nginx-mode nginx-indent-level)
     (nxml-mode nxml-child-indent (nxml-attribute-indent . 2))
     (objc-mode c-basic-offset)
@@ -197,7 +228,7 @@ overwrite \"indent_style\" property when current `major-mode' is a
     (ps-mode ps-mode-tab)
     (pug-mode pug-tab-width)
     (puppet-mode puppet-indent-level)
-    (python-mode . editorconfig-set-indentation/python-mode)
+    (python-mode . editorconfig-set-indentation-python-mode)
     (ruby-mode ruby-indent-level)
     (rust-mode rust-indent-offset)
     (scala-mode scala-indent:step)
@@ -207,6 +238,7 @@ overwrite \"indent_style\" property when current `major-mode' is a
     (slim-mode slim-indent-offset)
     (tcl-mode tcl-indent-level
               tcl-continued-indent-level)
+    (terra-mode terra-indent-level)
     (typescript-mode typescript-indent-level)
     (verilog-mode verilog-indent-level
                   verilog-indent-level-behavioral
@@ -236,23 +268,18 @@ If INDENT-SPEC-LIST is provided, each element of it must have one of the
 following forms:
 
  1. VARIABLE
-
     It means (VARIABLE . 1).
 
  2. (VARIABLE . SPEC)
-
     Setting VARIABLE according to the type of SPEC:
 
       - Integer
-
         The value is (* SPEC INDENT-SIZE);
 
       - Function
-
         The value is (funcall SPEC INDENT-SIZE);
 
       - Any other type.
-
         The value is SPEC.
 
 NOTE: Only the **buffer local** value of VARIABLE will be set."
@@ -265,15 +292,17 @@ NOTE: Only the **buffer local** value of VARIABLE will be set."
   "0.5")
 
 (defcustom editorconfig-exclude-modes ()
-  "List of major mode symbols not to apply properties."
+  "Modes in which `editorconfig-mode-apply' will not run."
   :type '(repeat (symbol :tag "Major Mode"))
   :group 'editorconfig)
 
 (defcustom editorconfig-exclude-regexps
   (list (eval-when-compile
           (rx string-start (or "http" "https" "ftp" "sftp" "rsync") ":")))
-  "List of buffer filename prefix regexp patterns not to apply
-properties."
+  "List of regexp for buffer filenames `editorconfig-mode-apply' will not run.
+
+When variable `buffer-file-name' matches any of the regexps, then
+`editorconfig-mode-apply' will not do its work."
   :type '(repeat string)
   :group 'editorconfig)
 
@@ -286,10 +315,13 @@ Otherwise, use `delete-trailing-whitespace'."
   :group 'editorconfig)
 
 (defvar editorconfig-properties-hash nil
-  "Hash object of EditorConfig properties for current buffer.
+  "Hash object of EditorConfig properties that was enabled for current buffer.
 Set by `editorconfig-apply' and nil if that is not invoked in
 current buffer yet.")
 (make-variable-buffer-local 'editorconfig-properties-hash)
+(put 'editorconfig-properties-hash
+     'permanent-local
+     t)
 
 (defvar editorconfig-lisp-use-default-indent nil
   "Selectively ignore the value of indent_sizefor Lisp files.
@@ -303,14 +335,14 @@ number - `lisp-indent-offset' is not set only if indent_size is
          `lisp-indent-offset'will not be set only if indent_size is 2.")
 
 (defconst editorconfig-unset-value "unset"
-  "String used to unset properties in .editorconfig .")
+  "String of value used to unset properties in .editorconfig .")
 
 (defun editorconfig-string-integer-p (string)
   "Return non-nil if STRING represents integer."
   (and (stringp string)
        (string-match-p "\\`[0-9]+\\'" string)))
 
-(defun editorconfig-set-indentation/python-mode (size)
+(defun editorconfig-set-indentation-python-mode (size)
   "Set `python-mode' indent size to SIZE."
   (set (make-local-variable (if (or (> emacs-major-version 24)
                                     (and (= emacs-major-version 24)
@@ -322,7 +354,7 @@ number - `lisp-indent-offset' is not set only if indent_size is
   (when (boundp 'py-indent-offset)
     (set (make-local-variable 'py-indent-offset) size)))
 
-(defun editorconfig-set-indentation/latex-mode (size)
+(defun editorconfig-set-indentation-latex-mode (size)
   "Set `latex-mode' indent size to SIZE."
   (set (make-local-variable 'tex-indent-basic) size)
   (set (make-local-variable 'tex-indent-item) size)
@@ -411,8 +443,10 @@ number - `lisp-indent-offset' is not set only if indent_size is
                                      nil t))))
 
 (defun editorconfig-set-trailing-nl (final-newline)
-  "Set up requiring final newline (`require-final-newline' and
-`mode-require-final-newline') by FINAL-NEWLINE."
+  "Set up requiring final newline by FINAL-NEWLINE.
+
+This function will set `require-final-newline' and `mode-require-final-newline'
+to non-nil when FINAL-NEWLINE is true."
   (cond
    ((equal final-newline "true")
     ;; keep prefs around how/when the nl is added, if set - otherwise add on save
@@ -454,6 +488,24 @@ number - `lisp-indent-offset' is not set only if indent_size is
              (> (string-to-number length) 0))
     (setq fill-column (string-to-number length))))
 
+(defvar editorconfig-file-type-emacs-whitelist
+  (append (mapcar 'car
+                  editorconfig-indentation-alist)
+          '(conf-mode))
+  "List of known `major-mode' that can be used for file_type_emacs value.")
+
+;; Emacs<26 does not have provided-mode-derived-p
+(defun editorconfig--provided-mode-derived-p (mode &rest modes)
+  "Non-nil if MODE is derived from one of MODES.
+Uses the `derived-mode-parent' property of the symbol to trace backwards.
+If you just want to check `major-mode', use `derived-mode-p'."
+  (if (fboundp 'provided-mode-derived-p)
+      (apply 'provided-mode-derived-p mode modes)
+    (while (and (not (memq mode modes))
+                (setq mode (get mode 'derived-mode-parent))))
+    mode))
+
+
 (defun editorconfig-set-major-mode-from-name (filetype)
   "Set buffer `major-mode' by FILETYPE.
 
@@ -465,7 +517,12 @@ FILETYPE should be s string like `\"ini\"`, if not nil or empty string."
                                    "-mode")))))
     (when mode
       (if (fboundp mode)
-          (editorconig-apply-major-mode-safely mode)
+          (if (apply 'editorconfig--provided-mode-derived-p mode
+                     editorconfig-file-type-emacs-whitelist)
+              (editorconfig-apply-major-mode-safely mode)
+            (display-warning :error (format "Major-mode `%S' is not listed in `%S'"
+                                            mode
+                                            'editorconfig-file-type-emacs-whitelist)))
         (display-warning :error (format "Major-mode `%S' not found"
                                         mode))
         nil))))
@@ -477,7 +534,7 @@ FILETYPE should be s string like `\"ini\"`, if not nil or empty string."
      'permanent-local
      t)
 
-(defun editorconig-apply-major-mode-safely (mode)
+(defun editorconfig-apply-major-mode-safely (mode)
   "Set `major-mode' to MODE.
 Normally `editorconfig-apply' will be hooked so that it runs when changing
 `major-mode', so there is a possibility that MODE is called infinitely if
@@ -526,7 +583,7 @@ EXT should be a string like `\"ini\"`, if not nil or empty string."
     (let ((mode (editorconfig--find-mode-from-ext ext
                                                   buffer-file-name)))
       (if mode
-            (editorconig-apply-major-mode-safely mode)
+          (editorconfig-apply-major-mode-safely mode)
         (display-warning :error (format "Major-mode for `%s' not found"
                                         ext))
         nil))))
@@ -557,8 +614,9 @@ EXT should be a string like `\"ini\"`, if not nil or empty string."
             (puthash key val properties)))))))
 
 (defun editorconfig-get-properties-from-exec ()
-  "Get EditorConfig properties of current buffer by calling
-`editorconfig-exec-path'."
+  "Get EditorConfig properties of current buffer.
+
+This function uses value of `editorconfig-exec-path' to get properties."
   (if (executable-find editorconfig-exec-path)
       (editorconfig-parse-properties (editorconfig-call-editorconfig-exec))
     (error "Unable to find editorconfig executable")))
@@ -575,40 +633,11 @@ It calls `editorconfig-get-properties-from-exec' if
     (require 'editorconfig-core)
     (editorconfig-core-get-properties-hash)))
 
-;;;###autoload
-(defun editorconfig-find-current-editorconfig ()
-  "Find the closest .editorconfig file for current file."
-  (interactive)
-  (eval-and-compile (require 'editorconfig-core))
-  (let ((file (editorconfig-core-get-nearest-editorconfig
-               default-directory)))
-    (when file
-      (find-file file))))
-
-;;;###autoload
-(defun editorconfig-display-current-properties ()
-  "Display EditorConfig properties extracted for current buffer."
-  (interactive)
-  (if editorconfig-properties-hash
-      (let (
-            (buf (get-buffer-create "*EditorConfig Properties*"))
-            (file buffer-file-name)
-            (props editorconfig-properties-hash))
-        (with-current-buffer buf
-          (erase-buffer)
-          (insert (format "# EditorConfig for %s\n" file))
-          (maphash (lambda (k v)
-                     (insert (format "%S = %s\n" k v)))
-                   props))
-        (display-buffer buf))
-    (message "Properties are not applied to current buffer yet.")
-    nil))
-
 ;;;###autoload
 (defun editorconfig-apply ()
-  "Apply EditorConfig properties for current buffer.
-This function ignores `editorconfig-exclude-modes' and always
-applies available properties."
+  "Get and apply EditorConfig properties to current buffer.
+This function ignores `editorconfig-exclude-modes' and
+`editorconfig-exclude-regexps', and always applies available properties."
   (interactive)
   (when buffer-file-name
     (condition-case err
@@ -650,9 +679,10 @@ applies available properties."
                         :error)))))
 
 (defun editorconfig-mode-apply ()
-  "Apply EditorConfig properties for current buffer.
-This function does the job only when the major mode is not listed
-in `editorconfig-exclude-modes'."
+  "Get and apply EditorConfig properties to current buffer.
+This function does nothing when the major mode is listed in
+`editorconfig-exclude-modes', or variable `buffer-file-name' matches
+any of regexps in `editorconfig-exclude-regexps'."
   (when (and major-mode
              (not (memq major-mode
                         editorconfig-exclude-modes))
@@ -662,8 +692,67 @@ in `editorconfig-exclude-modes'."
                            finally return nil)))
     (editorconfig-apply)))
 
+;;;###autoload
+(define-minor-mode editorconfig-mode
+  "Toggle EditorConfig feature.
+
+To disable EditorConfig in some buffers, modify
+`editorconfig-exclude-modes' or `editorconfig-exclude-regexps'."
+  :global t
+  :lighter editorconfig-mode-lighter
+  ;; See https://github.com/editorconfig/editorconfig-emacs/issues/141 for why
+  ;; not `after-change-major-mode-hook'
+  (dolist (hook '(change-major-mode-after-body-hook
+                  read-only-mode-hook
+                  ;; Some modes call `kill-all-local-variables' in their init
+                  ;; code, which clears some values set by editorconfig.
+                  ;; For those modes, editorconfig-apply need to be called
+                  ;; explicitly through their hooks.
+                  rpm-spec-mode-hook
+                  ))
+    (if editorconfig-mode
+        (add-hook hook 'editorconfig-mode-apply)
+      (remove-hook hook 'editorconfig-mode-apply))))
+
+
+;; Tools
+;; Some useful commands for users, not required for EditorConfig to work
+
+;;;###autoload
+(defun editorconfig-find-current-editorconfig ()
+  "Find the closest .editorconfig file for current file."
+  (interactive)
+  (eval-and-compile (require 'editorconfig-core))
+  (let ((file (editorconfig-core-get-nearest-editorconfig
+               default-directory)))
+    (when file
+      (find-file file))))
+
+;;;###autoload
+(defun editorconfig-display-current-properties ()
+  "Display EditorConfig properties extracted for current buffer."
+  (interactive)
+  (if editorconfig-properties-hash
+      (let (
+            (buf (get-buffer-create "*EditorConfig Properties*"))
+            (file buffer-file-name)
+            (props editorconfig-properties-hash))
+        (with-current-buffer buf
+          (erase-buffer)
+          (insert (format "# EditorConfig for %s\n" file))
+          (maphash (lambda (k v)
+                     (insert (format "%S = %s\n" k v)))
+                   props))
+        (display-buffer buf))
+    (message "Properties are not applied to current buffer yet.")
+    nil))
+;;;###autoload
+(defalias 'describe-editorconfig-properties
+  'editorconfig-display-current-properties)
+
+;;;###autoload
 (defun editorconfig-format-buffer()
-  "Format buffer according to .editorconfig indent_style and indent_width"
+  "Format buffer according to .editorconfig indent_style and indent_width."
   (interactive)
   (if (string= (gethash 'indent_style editorconfig-properties-hash) "tab")
       (tabify (point-min) (point-max)))
@@ -672,21 +761,46 @@ in `editorconfig-exclude-modes'."
   (indent-region (point-min) (point-max)))
 
 
+
+;; (defconst editorconfig--version
+;;   (eval-when-compile
+;;     (require 'lisp-mnt)
+;;     (declare-function lm-version "lisp-mnt" nil)
+;;     (lm-version))
+;;   "EditorConfig version.")
+
+(declare-function find-library-name "find-func" (library))
+(declare-function lm-version "lisp-mnt" nil)
+
 ;;;###autoload
-(define-minor-mode editorconfig-mode
-  "Toggle EditorConfig feature.
-When enabled EditorConfig properties will be applied to buffers
-when first visiting files or changing major modes if the major
-mode is not listed in `editorconfig-exclude-modes'."
-  :global t
-  :lighter editorconfig-mode-lighter
-  ;; See https://github.com/editorconfig/editorconfig-emacs/issues/141 for why
-  ;; not `after-change-major-mode-hook'
-  (dolist (hook '(change-major-mode-after-body-hook
-                  read-only-mode-hook))
-    (if editorconfig-mode
-        (add-hook hook 'editorconfig-mode-apply)
-      (remove-hook hook 'editorconfig-mode-apply))))
+(defun  editorconfig-version (&optional show-version)
+  "Get EditorConfig version as string.
+
+If called interactively or if SHOW-VERSION is non-nil, show the
+version in the echo area and the messages buffer."
+  (interactive (list t))
+  (let* ((version
+          (with-temp-buffer
+            (require 'find-func)
+            (insert-file-contents (find-library-name "editorconfig"))
+            (require 'lisp-mnt)
+            (lm-version)))
+         (pkg
+          (and (require 'package nil t)
+               (cadr (assq 'editorconfig
+                           package-alist))))
+         (pkg-version
+          (and pkg
+               (package-version-join (package-desc-version pkg))))
+         (version-full (if (and pkg-version
+                                (not (string= version
+                                              pkg-version)))
+                           (concat version "-" pkg-version)
+                         version)))
+    (when show-version
+      (message "EditorConfig Emacs v%s"
+               version-full))
+    version-full))
 
 (provide 'editorconfig)
 
